/* Lexical scanner to find out what kind of SGML document we are dealing
 * with.  This is not general: it just returns the name of the top level
 * element and a flag indicating whether a DTD needs to be prefixed before
 * processing with sgmls.
 *
 * Copyright (C) 1993, 1994, Gary Houston.
 * This program may be copied and modified under the terms of the 
 * GNU General Public License version 2.
 *
 */
/* The LHS must be case insensitive: use flex -i to generate.  */

%x comment
%x innercomment
%x sgmldec
%x sgmlcomment
%x doctype
%{
#define YY_DECL int auto_detect(void)

#include <stdlib.h>
#include <string.h>
#include "gf.h"

	/* Set to 1 if a SGML declaration is found.  */
	int detect_got_SGML;
        /* Set to 1 if a DOCTYPE declaration is found.  */
	int detect_got_DTD;
	/* This returns the document type name.  Space will be allocated
	 * automatically.
	 */
	char *detect_name = NULL;

%}

%%
%{
  detect_got_SGML = 0;
  detect_got_DTD = 0;
%}
[ \t\n]+ 		/* Eat up whitespace.  */
"<!--"			BEGIN(comment);
<comment>[^-]+		/* Eat up comment.  */
<comment>"-"[^-]        /* Eat up dash in comment.  */
<comment><<EOF>>	return -1;
<comment>"--"" "*">"	BEGIN(INITIAL);
<comment>"--"           BEGIN(innercomment);  /* Deal with SGML weirdness. */
<innercomment>[^-]+     /* Eat up weird comment.  */
<innercomment>"-"[^-]   /* Eat up dash in weird comment.  */
<innercomment>"--"      BEGIN(comment);
<innercomment><<EOF>>	return -1;
"<!SGML"		{
  			  detect_got_SGML = 1;
			  BEGIN(sgmldec);
			}
<sgmldec>[^\>\-]+	/* Eat up SGML declaration.  */
<sgmldec>"--"		BEGIN(sgmlcomment);  /* Comment in SGML declaration. */
<sgmldec>"-"[^-]	/* Eat up SGML dash.  */
<sgmldec><<EOF>>	return -1;
<sgmldec>">"		BEGIN(INITIAL);
<sgmlcomment>[^\-]+	/* Eat up SGML comment.  */
<sgmlcomment>"-"[^-]	/* Eat up dash in SGML comment.  */
<sgmlcomment>"--"	BEGIN(sgmldec);  /* End of SGML comment.  */
<sgmlcomment><<EOF>>	return -1;
"<!DOCTYPE"		{
  			  detect_got_DTD = 1;
			  BEGIN(doctype);
			}
<doctype>[ \t\n]+	/* Eat up whitespace.  */
<doctype>[A-Z]+		{
  			  if (detect_name == NULL)
			    detect_name = galloc(yyleng + 1);
  			  else
			    detect_name = grealloc(detect_name, yyleng + 1);
			  strcpy(detect_name, yytext);
			  return 2;
			}
<doctype>[^A-Z \n\t]	return -1; /* Should have returned already. */
"<"[A-Z]*">"		{
  			  if (detect_got_SGML && !detect_got_DTD) {
			    return -1;
			  }
			  /* Don't copy < or >.  */
  			  detect_name = galloc(yyleng - 1);
			  strncpy(detect_name, yytext + 1, yyleng - 2);
			  detect_name[yyleng - 2] = '\0';
			  return 3;
			}
[^<]			return -1;
"<"[^!]			return -1;
